//! The ableOS kernel.

#![feature(
    abi_x86_interrupt,
    alloc_error_handler,
    inline_const,
    panic_info_message,
    pointer_is_aligned,
    prelude_import,
    ptr_sub_ptr
)]
#![no_std]

extern crate alloc;

mod allocator;
mod arch;
pub mod interp;
mod kmain;
mod logger;
mod memory;
mod task;

use versioning::Version;

/// Kernel's version
pub const VERSION: Version = Version {
    major: 0,
    minor: 2,
    patch: 0,
};

#[panic_handler]
fn panic(info: &core::panic::PanicInfo) -> ! {
    if let Some(loc) = info.location() {
        let _ = crate::arch::log(format_args!(
            "Location: {}: {}, {}\r\n",
            loc.file(),
            loc.line(),
            loc.column()
        ));
    }

    if let Some(msg) = info.message() {
        let _ = crate::arch::log(format_args!("{msg}\r\n"));
    }

    loop {}
}
